#ifndef BASE64TX_H
#define BASE64TX_H

#include "uart0_no_rx.h"
#include <stdint.h>

extern uint8_t base64_buffer[32];
extern uint8_t bits_in_buffer;

// Takes value as pointer to little-endian data, and pushes the lowest bits
// to Base64 buffer. Doesn't check for buffer overflow, and doesn't mask bits!
void push_value(uint32_t value, uint8_t bits);

// Converts the data in the buffer to a Base64 stream and sends it via uart:
void send_buffer_base64_no_newline(void);

// Sends the buffer via UART as one Base64 line (without =):
static inline void send_buffer_base64(void)
{
    send_buffer_base64_no_newline();
    uart_putc('\n');
}

// Tells the buffer to forget about all data:
void reset_buffer(void);

// Encodes 6 bit (right aligned) to one Base64 char:
static inline void send_base64_char(uint8_t value)
{
    value &= 63;
    
    // For Uuencoding use:
    //uart_putc(value + 32);
    
    if (value < 26) {
        uart_putc(value + 'A');
    } else if (value < 52) {
        uart_putc(value - 26 + 'a');
    } else if (value < 62) {
        uart_putc(value - 52 + '0');
    } else if (value == 62) {
        uart_putc('+');
    } else {
        uart_putc('/');
    }
}

#endif // BASE64TX_H
